from typing import Optional

from subsearch.data import __video_directory__
from subsearch.scraper import opensubtitles_soup
from subsearch.utils import log
from subsearch.utils.string_parser import SearchParameters


# decides what to do with all the scrape data
def scrape(param: SearchParameters, lang: str, hi: bool):
    """
    Scrape opensubtitles for subtitles using the given parameters.

    Args:
        param (_type_): url_opensubtitles, file_hash
        lang (str): desired language
        hi (str): if hearing impaired subtitles are desired

    Returns:
        list | None: download links for the subtitle
    """
    to_be_downloaded: Optional[list[str]] = opensubtitles_soup.search_for_hash(param.url_opensubtitles, lang, hi)
    if to_be_downloaded is None:
        if param.show_bool:
            log.output(f"No TV-series found matching hash {param.file_hash}")
        else:
            log.output(f"No movies found matching hash {param.file_hash}")
        return None
    else:
        download_info = []
        log.output(f"Preparing  hash {param.file_hash} for download")
        for current_num, dl_url in enumerate(to_be_downloaded):
            total_num = len(to_be_downloaded)
            current_num += 1
            file_path = f"{__video_directory__}\\opensubtitles_{current_num}.zip"
            x = file_path, dl_url, current_num, total_num
            download_info.append(x)
        log.output(f"Done with tasks")
        return download_info



import os
import socket
import sys
import winreg
from typing import Literal

from subsearch.data import __icons__, __root__
from subsearch.utils import current_user

COMPUTER_NAME = socket.gethostname()
ASTERISK_PATH = "Software\\Classes\\*"
SHELL_PATH = "Software\\Classes\\*\\shell"
SUBSEARCH_PATH = "Software\\Classes\\*\\shell\\0.SubSearch"
COMMAND_PATH = "Software\\Classes\\*\\shell\\0.SubSearch\\command"


def write_keys() -> None:
    """
    Write the necessary keys to the registry
    """
    with winreg.ConnectRegistry(COMPUTER_NAME, winreg.HKEY_CURRENT_USER) as hkey:
        # open key, with write permission
        with winreg.OpenKey(hkey, ASTERISK_PATH, 0, winreg.KEY_WRITE) as sk:
            winreg.CreateKey(sk, "shell")
        with winreg.OpenKey(hkey, SHELL_PATH, 0, winreg.KEY_WRITE) as sk:
            winreg.CreateKey(sk, "0.SubSearch")
        with winreg.OpenKey(hkey, SUBSEARCH_PATH, 0, winreg.KEY_WRITE) as sk:
            winreg.CreateKey(sk, "command")


def write_all_valuex() -> None:
    write_valuex("SubSearch")
    write_valuex("icon")
    write_valuex("appliesto")
    write_valuex("command")


def write_valuex(key: str) -> None:
    """
    Write valuex into the different keys

    """
    # decide in which registry key to write the value
    if key == "SubSearch":
        key_type = SUBSEARCH_PATH
        value_name = ""
        value = "SubSearch"
    if key.lower() == "icon":
        key_type = SUBSEARCH_PATH
        value_name = "Icon"
        value = get_icon_value()
    elif key.lower() == "appliesto":
        key_type = SUBSEARCH_PATH
        value_name = "AppliesTo"
        value = get_appliesto_value()
    elif key.lower() == "command":
        key_type = COMMAND_PATH
        value_name = ""
        value = get_command_value()
    open_write_valuex(key_type, value_name, value)


def open_write_valuex(sub_key: str, value_name: str, value: str) -> None:
    """
    Connects to the registry, opens sub-key then set valuex with value
    """
    try:
        # connect to Computer_NAME\HKEY_CURRENT_USER\
        with winreg.ConnectRegistry(COMPUTER_NAME, winreg.HKEY_CURRENT_USER) as hkey:
            # open key, with write permission
            with winreg.OpenKey(hkey, sub_key, 0, winreg.KEY_WRITE) as sk:
                # write valuex to matching value name
                winreg.SetValueEx(sk, value_name, 0, winreg.REG_SZ, value)
    except FileNotFoundError:
        pass


def get_command_value() -> str:
    """
    Get the correct command value
    """
    # get latest json value from file
    from subsearch.utils import raw_config

    if current_user.check_is_exe():
        value = f'"{sys.argv[0]}" "%1"'
        # if SubSearch is compiled we dont need anything besides this
    elif current_user.check_is_exe() is False:
        show_terminal = raw_config.get("show_terminal")
        # gets the location to the python executable
        python_path = os.path.dirname(sys.executable)
        # sys.args[-1] is going to be the path to the file we right clicked on
        # import_sys = "import sys; media_file_path = sys.argv[-1];"
        set_title = "import ctypes; ctypes.windll.kernel32.SetConsoleTitleW('SubSearch');"
        # gets the path of the root directory of subsearch
        set_wd = f"import os; os.chdir(r'{__root__}');"
        import_main = "import subsearch; subsearch.main()"
        if show_terminal is True:
            value = f'{python_path}\python.exe -c "{set_title} {set_wd} {import_main}" "%1"'
        if show_terminal is False:
            value = f'{python_path}\pythonw.exe -c "{set_title} {set_wd} {import_main}" "%1"'

    return value


def get_icon_value() -> str:
    """
    Check config.json cm_icon value

    """
    from subsearch.utils import raw_config

    show_icon: str = raw_config.get("cm_icon")
    icon_path = os.path.join(__icons__, "16.ico")
    if show_icon:
        return icon_path
    else:
        return ""


def get_appliesto_value() -> str:
    # get latest json value from file
    from subsearch.utils import raw_config

    file_ext = raw_config.get("file_ext")
    # for which file types to show the SubSearch context entry on
    value = '"subsearch.py" OR '
    for k, v in zip(file_ext.keys(), file_ext.values()):
        if v is True:
            value += "".join(f'"{k}" OR ')
    if value.endswith(" OR "):  # remove last OR
        value = value[:-4]

    return value


def remove_context_menu() -> None:
    with winreg.ConnectRegistry(COMPUTER_NAME, winreg.HKEY_CURRENT_USER) as hkey:
        with winreg.OpenKey(hkey, SUBSEARCH_PATH, 0, winreg.KEY_WRITE) as sk:
            winreg.DeleteKey(sk, "command")

    with winreg.ConnectRegistry(COMPUTER_NAME, winreg.HKEY_CURRENT_USER) as hkey:
        with winreg.OpenKey(hkey, SHELL_PATH, 0, winreg.KEY_WRITE) as sk:
            winreg.DeleteKey(sk, "0.SubSearch")


# write keys to registry then write values
def add_context_menu() -> None:
    write_keys()
    write_all_valuex()



import re
from typing import Optional

import cloudscraper

from subsearch.data import __version__

SCRAPER = cloudscraper.create_scraper(browser={"browser": "chrome", "platform": "android", "desktop": False})


def check_for_updates() -> str:
    source = SCRAPER.get(
        "https://raw.githubusercontent.com/vagabondHustler/subsearch/main/src/subsearch/data/__version__.py"
    )
    scontent = source.content
    _string = str(scontent)
    latest_version = "".join(re.findall('^.*["]([0-9]*)\.([0-9]*)\.([0-9]*)["]', _string)[0])
    return latest_version


def is_new_version_available() -> tuple[bool, Optional[str]]:
    """
    Compare local version with latest version on github

    Returns:
        tuple[bool, str]: True, None if local version is less than repo, False, "newer" if local version is greater than repo, False, else False None
    """

    local_version = "".join(re.findall("([0-9]*)\.([0-9]*)\.([0-9]*)", __version__)[0])
    repo_version = check_for_updates()
    if local_version < repo_version:
        return True, None
    if local_version > repo_version:
        return False, "newer"
    return False, None



from .__set_data__ import Paths

__root__: str = getattr(Paths, "root")
__data__: str = getattr(Paths, "data")
__gui__: str = getattr(Paths, "gui")
__scraper__: str = getattr(Paths, "scraper")
__utils__: str = getattr(Paths, "utils")
__icons__: str = getattr(Paths, "icons")
__buttons__: str = getattr(Paths, "buttons")



from .__set_data__ import VideoFile

__video_name__: str = getattr(VideoFile, "name")
__video_ext__: str = getattr(VideoFile, "ext")
__video_path__: str = getattr(VideoFile, "path")
__video_directory__: str = getattr(VideoFile, "directory")



import ctypes
import os
import sys
import time

from subsearch.data import (
    __version__,
    __video_directory__,
    __video_name__,
    __video_path__,
)
from subsearch.gui import widget_download, widget_settings
from subsearch.scraper import opensubtitles, subscene
from subsearch.utils import (
    current_user,
    file_manager,
    log,
    raw_config,
    raw_registry,
    string_parser,
)

PACKAGEPATH = os.path.abspath(os.path.dirname(__file__))
HOMEPATH = os.path.dirname(PACKAGEPATH)
sys.path.append(HOMEPATH)
sys.path.append(PACKAGEPATH)


class SubSearch:
    def __init__(self) -> None:
        """
        Setup and gather all available parameters
        """
        self.start = time.perf_counter()
        self.scrape_opensubtitles = None
        self.scrape_subscene = None
        ctypes.windll.kernel32.SetConsoleTitleW(f"SubSearch - {__version__}")
        if current_user.got_key() is False:
            raw_config.set_default_json()
            raw_registry.add_context_menu()
        self.show_terminal = False if current_user.check_is_exe() else raw_config.get("show_terminal")
        self.file_exist = True if __video_name__ is not None else False
        self.lang, self.lang_code2 = raw_config.get("language")
        self.hearing_impared = raw_config.get("hearing_impaired")
        self.pct = raw_config.get("percentage")
        self.show_dl_window = raw_config.get("show_download_window")
        if self.file_exist:
            file_hash = file_manager.get_hash(__video_path__)
            self.param = string_parser.get_parameters(__video_name__, file_hash, self.lang, self.lang_code2)
            log.parameters(self.param, self.lang, self.lang_code2, self.hearing_impared, self.pct)
            if " " in __video_name__:
                log.output("[Warning: Filename contains spaces]")
        if self.file_exist is False:
            widget_settings.show_widget()

    def opensubtitles_scrape(self) -> None:
        """
        Scrape on opensubtitles by filehash
        """
        if self.file_exist:
            log.output("")
            log.output("[Searching on opensubtitles]")
            self.scrape_opensubtitles = opensubtitles.scrape(self.param, self.lang, self.hearing_impared)

    def subscene_scrape(self) -> None:
        """
        Scrape subscene from parsing filename of the video
        """
        if self.file_exist:
            log.output("")
            log.output("[Searching on subscene]")
            self.scrape_subscene = subscene.scrape(
                self.param, self.lang, self.lang_code2, self.hearing_impared, self.pct, self.show_dl_window
            )

    def process_files(self) -> None:
        """
        Download zip files containing the .srt files, extract, rename and clean up tmp files
        """
        if self.file_exist is False:
            return None

        if self.scrape_opensubtitles is not None:
            log.output("")
            log.output("[Downloading from Opensubtitles]")
            for item in self.scrape_opensubtitles:
                file_manager.download_zip(item)

        if self.scrape_subscene is not None:
            log.output("")
            log.output("[Downloading from Subscene]")
            for item in self.scrape_subscene:
                file_manager.download_zip(item)

        if self.scrape_opensubtitles is None and self.scrape_subscene is None:
            dl_data = f"{__video_directory__}\\__subsearch__dl_data.tmp"
            if self.show_dl_window is True and os.path.exists(dl_data):
                widget_download.show_widget()
                file_manager.clean_up(__video_directory__, dl_data)

        log.output("")
        log.output("[Procsessing files]")
        file_manager.extract_zips(__video_directory__, ".zip")
        file_manager.clean_up(__video_directory__, ".zip")
        file_manager.clean_up(__video_directory__, ").nfo")
        file_manager.rename_best_match(f"{self.param.release}.srt", __video_directory__, ".srt")

    def end(self) -> None:
        """
        Stop pref counter, log elapsed time and keep the terminal open if show_terminal is True
        """
        elapsed = time.perf_counter() - self.start
        log.output("")
        log.output(f"Finished in {elapsed} seconds")

        if self.show_terminal and current_user.check_is_exe() is False:
            input("Ctrl + c or Enter to exit")


def con() -> None:
    r"""
    Usages: subsearch [OPTIONS]

    Options:
        --settings                                  Open the GUI settings menu

        --registry-key [add, del]                   Edit the registry
                                                    add: adds the context menu  / replaces the context menu with default values
                                                    del: deletes the context menu
                                                    e.g: subsearch --registry-key add

        --help                                      Prints usage information
    """
    if sys.argv[-1].endswith("subsearch"):
        print(con.__doc__)
        return
    else:
        for num, arg in enumerate(sys.argv[1:], 1):
            if arg.startswith("--settings"):
                sys.argv.pop(num)  # pop settings argument
                widget_settings.show_widget()
                break
            elif arg.startswith("--registry-key") or arg.startswith("--add-key"):
                if sys.argv[num + 1] == "add":
                    sys.argv.pop(num)  # pop registry argument
                    sys.argv.pop(num)  # pop add argument
                    raw_registry.add_context_menu()
                    break
                elif sys.argv[num + 1] == "del":
                    sys.argv.pop(num)  # pop registry argument
                    sys.argv.pop(num)  # pop del argument
                    raw_registry.remove_context_menu()
                    break
            elif arg.startswith("--help"):
                sys.argv.pop(num)  # pop help argument
                print(con.__doc__)
                break
            elif len(sys.argv[1:]) == num:
                print("Invalid argument")
                print(con.__doc__)


def main() -> None:
    for i in sys.argv:
        if i.startswith("--"):
            con()
            return None

    ss = SubSearch()
    ss.opensubtitles_scrape()
    ss.subscene_scrape()
    ss.process_files()
    ss.end()


if __name__ == "__main__":
    main()



import ctypes
import os
import tkinter as tk
from tkinter import Label, StringVar

from subsearch.data import __buttons__
from subsearch.gui import tkdata

TKCOLOR = tkdata.Color()
TKFONT = tkdata.Font()
TKWINDOW = tkdata.Window()

GWL_EXSTYLE = -20
WS_EX_APPWINDOW = 0x00040000
WS_EX_TOOLWINDOW = 0x00000080


def buttons(btn: str):
    return os.path.join(__buttons__, btn)


# create custom labels and buttons in grid
class Create(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.configure(bg=TKCOLOR.dark_grey)

    # create a basic label
    def label(
        self,
        bg=TKCOLOR.dark_grey,
        fg=TKCOLOR.white_grey,
        text=None,
        textvar=None,
        row=1,
        col=1,
        anchor=None,
        sticky=None,
        font=TKFONT.cas8b,
        padx=2,
        pady=2,
    ):
        _label = tk.Label(self, text=text, textvariable=textvar, font=font, fg=fg, anchor=anchor)
        _label.configure(bg=bg, fg=fg, font=font)
        _label.grid(row=row, column=col, sticky=sticky, padx=padx, pady=pady)
        return _label

    # create a basic button
    def button(
        self,
        widget_refrence=None,
        bg=TKCOLOR.light_black,
        abgc=TKCOLOR.orange,
        bge=TKCOLOR.black,
        fg=TKCOLOR.white_grey,
        fge=TKCOLOR.orange,
        text=None,
        height=2,
        width=10,
        bd=0,
        row=1,
        col=3,
        sticky=None,
        font=TKFONT.cas8b,
        padx=5,
        pady=2,
        bind_to=None,
        tip_show=False,
        tip_text="",
    ):
        _button = tk.Button(self, text=text, height=height, width=width, bd=bd)
        _button.configure(activebackground=abgc, bg=bg, fg=fg, font=font)
        _button.grid(row=row, column=col, padx=padx, pady=pady, sticky=sticky)
        _button.bind("<Button-1>", bind_to)
        tip = ToolTip(_button, _button, tip_text) if tip_show else None

        def button_enter(self):
            _button.configure(bg=bge, fg=fge, font=font)
            tip.show() if tip_show else None

        def button_leave(self):
            _button.configure(bg=bg, fg=fg, font=font)
            tip.hide() if tip is not None else None

        _button.bind("<Enter>", button_enter)
        _button.bind("<Leave>", button_leave)
        return _button


# replace the regular windows-style title bar with a custom one
class CustomTitleBar(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self._offsetx = 0
        self._offsety = 0
        self.parent = parent
        self.after(10, self.remove_old_tb(), self.parent)
        self.parent.focus_force()
        self.parent.overrideredirect(True)
        # place new bar at top of window
        self.bar = tk.Frame(height=37, width=TKWINDOW.width, bg=TKCOLOR.light_black)
        self.bar.place(x=0, y=0)
        # place SubSearch in bar
        self.subsearch_label = tk.Label(
            self.bar,
            text="SubSearch",
            bg=TKCOLOR.light_black,
            fg=TKCOLOR.white_grey,
            font=TKFONT.cas10b,
            justify="center",
            anchor="w",
        )
        self.subsearch_label.place(bordermode="inside", relx=0, y=18, anchor="w")
        # png paths
        self.exit_path = buttons("exit.png")
        self.tab_path = buttons("tab.png")
        self.exit_grey_path = buttons("exit_grey.png")
        self.tab_grey_path = buttons("tab_grey.png")
        self.max_ina_path = buttons("maximize_inactive.png")
        # PhotoImages
        self.tab_png = tk.PhotoImage(file=self.tab_path)
        self.tab_grey_png = tk.PhotoImage(file=self.tab_grey_path)
        self.exit_png = tk.PhotoImage(file=self.exit_path)
        self.exit_grey_png = tk.PhotoImage(file=self.exit_grey_path)
        self.max_ina_png = tk.PhotoImage(file=self.max_ina_path)
        # place exit canvas and image
        self.exit = tk.Canvas(
            self.bar,
            width=37,
            height=37,
            bg=TKCOLOR.light_black,
            highlightthickness=0,
        )
        self.exit.place(relx=1, rely=0, anchor="ne")
        self.update_img(self.exit, self.exit_grey_png)
        # place maximize canvas and image
        self.maximize = tk.Canvas(
            self.bar,
            width=37,
            height=37,
            bg=TKCOLOR.light_black,
            highlightthickness=0,
        )
        self.maximize.place(x=TKWINDOW.width - 37, rely=0, anchor="ne")
        self.update_img(self.maximize, self.max_ina_png)
        # place tab canvas and image
        self.tab = tk.Canvas(
            self.bar,
            width=37,
            height=37,
            bg=TKCOLOR.light_black,
            highlightthickness=0,
        )
        self.tab.place(x=TKWINDOW.width - 74, rely=0, anchor="ne")
        self.update_img(self.tab, self.tab_grey_png)
        # binds
        # bind label for dragging around window
        self.subsearch_label.bind("<Button-1>", self.tb_press)
        self.subsearch_label.bind("<B1-Motion>", self.tb_drag)
        # bind titlebar for dragging around window
        self.bar.bind("<Button-1>", self.tb_press)
        self.bar.bind("<B1-Motion>", self.tb_drag)
        # bind exit to close window and update png
        self.exit.bind("<Enter>", self.exit_enter)
        self.exit.bind("<Leave>", self.exit_leave)
        # bind tab to tab window and update png
        self.tab.bind("<Enter>", self.tab_enter)
        self.tab.bind("<Leave>", self.tab_leave)
        self.parent.bind("<Enter>", self.parent_enter)
        self.parent.bind("<Leave>", self.parent_leave)

        # hide 1px Frame
        self.place(relx=1, rely=1, anchor="n")
        self.configure(bg=TKCOLOR.green)

    def window_pos(self, parent, w: int = 80, h: int = 48):
        ws = parent.winfo_screenwidth()
        hs = parent.winfo_screenheight()

        x = int((ws / 2) - (w / 2))
        y = int((hs / 2) - (h / 2))
        value = f"{w}x{h}+{x}+{y}"
        return value

    def remove_old_tb(self):
        hwnd = ctypes.windll.user32.GetParent(self.parent.winfo_id())
        style = ctypes.windll.user32.GetWindowLongPtrW(hwnd, GWL_EXSTYLE)
        style = style & ~WS_EX_TOOLWINDOW
        style = style | WS_EX_APPWINDOW
        res = ctypes.windll.user32.SetWindowLongPtrW(hwnd, GWL_EXSTYLE, style)
        self.parent.withdraw()
        self.parent.after(100, lambda: self.parent.wm_deiconify())

    def tb_press(self, event):
        self._offsetx = self.parent.winfo_pointerx() - self.parent.winfo_rootx()
        self._offsety = self.parent.winfo_pointery() - self.parent.winfo_rooty()

    def tb_drag(self, event):
        x = self.winfo_pointerx() - self._offsetx
        y = self.winfo_pointery() - self._offsety
        self.parent.geometry(f"+{x}+{y}")

    def check_window_state(self, event):
        if event.widget == self.parent:
            if self.parent.state() == "iconic":
                pass
            if self.parent.state() == "normal":
                self.parent.overrideredirect(True)
                self.parent.attributes("-alpha", 1)
                self.parent.deiconify()

    def tabbing(self, event):
        self.parent.attributes("-topmost", False)
        self.parent.wm_attributes("-transparentcolor", TKCOLOR.black)
        self.parent.attributes("-alpha", 0.1)
        self.parent.overrideredirect(False)
        self.parent.iconify()
        self.parent.bind("<FocusIn>", self.check_window_state)

    def exit_release(self, event):
        self.parent.destroy()

    def exit_press(self, event):
        self.exit.configure(bg=TKCOLOR.dark_red)
        self.exit.bind("<ButtonRelease-1>", self.exit_release)

    def exit_enter(self, event):
        self.exit.configure(bg=TKCOLOR.red)
        self.update_img(self.exit, self.exit_png)
        self.exit.bind("<ButtonPress-1>", self.exit_press)

    def exit_leave(self, event):
        self.exit.configure(bg=TKCOLOR.light_black)
        self.update_img(self.exit, self.exit_grey_png)
        self.exit.unbind("<ButtonRelease-1>")

    def tab_press(self, event):
        self.tab.configure(bg=TKCOLOR.light_grey)
        self.tab.bind("<ButtonRelease-1>", self.tabbing)

    def tab_enter(self, event):
        self.tab.configure(bg=TKCOLOR.dark_grey)
        self.update_img(self.tab, self.tab_png)
        self.tab.bind("<ButtonPress-1>", self.tab_press)

    def tab_leave(self, event):
        self.tab.configure(bg=TKCOLOR.light_black)
        self.update_img(self.tab, self.tab_grey_png)
        self.tab.unbind("<ButtonRelease-1>")

    def parent_enter(self, event):
        self.parent.attributes("-topmost", False)

    def parent_leave(self, evnt):
        self.parent.attributes("-topmost", True)

    def update_img(self, canvas, img):
        canvas.delete("all")
        canvas.create_image(18, 18, image=img)
        canvas.photoimage = img


# create a custom border for window
class CustomBorder(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        csx = TKWINDOW.width
        csy = TKWINDOW.height
        self.canvas_border = tk.Canvas(parent, width=csx, height=csy, bg=TKCOLOR.light_black, borderwidth=0)
        self.canvas_border.place(relx=0.5, rely=0.5, anchor="center")
        self.canvas_bg = tk.Canvas(
            parent,
            width=csx - 4,
            height=csy - 4,
            bg=TKCOLOR.dark_grey,
            highlightthickness=0,
        )
        self.canvas_bg.place(relx=0.5, rely=0.5, anchor="center")

        self.configure(bg=TKCOLOR.light_black)


# get the window position so it can be placed in the center of the screen
class WindowPosition(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)

    def set(
        self,
        w=TKWINDOW.width,
        h=TKWINDOW.height,
        ws_value_offset=0,
        hs_value_offset=0,
    ):
        ws = self.winfo_screenwidth() + ws_value_offset
        hs = self.winfo_screenheight() + hs_value_offset
        x = int((ws / 2) - (w / 2))
        y = int((hs / 2) - (h / 2))
        value = f"{w}x{h}+{x}+{y}"
        return value


# pick a appropriate color for StringVar based on the string
class ColorPicker:
    def __init__(self, string_var: StringVar, clabel: Label, pct: int = -1):
        self.string_var = string_var
        self.clabel = clabel
        self.pct = pct
        self.pick()

    def pick(self):  # string boolean
        if self.string_var.get() == "True":
            self.clabel.configure(fg=TKCOLOR.green)
        elif self.string_var.get() == "False":
            self.clabel.configure(fg=TKCOLOR.red)
        elif self.string_var.get() == "Both":
            self.clabel.configure(fg=TKCOLOR.blue)
        # range
        elif self.pct in range(75, 100):
            self.clabel.configure(fg=TKCOLOR.green)
        elif self.pct in range(50, 75):
            self.clabel.configure(fg=TKCOLOR.green_brown)
        elif self.pct in range(25, 50):
            self.clabel.configure(fg=TKCOLOR.red_brown)
        elif self.pct in range(0, 25):
            self.clabel.configure(fg=TKCOLOR.red)


class ToolTip(tk.Toplevel):
    def __init__(self, parent, widget, *text):
        self.parent = parent
        self.widget = widget
        self.text = text

    def show(self):
        tk.Toplevel.__init__(self, self.parent)
        self.configure(background=TKCOLOR.light_black)
        # remove the standard window titlebar from the tooltip
        self.overrideredirect(True)
        # unpack *args and put each /n on a new line
        lines = "\n".join(self.text)
        frame = tk.Frame(self, background=TKCOLOR.light_black)
        label = tk.Label(
            frame,
            text=lines,
            background=TKCOLOR.dark_grey,
            foreground=TKCOLOR.white_grey,
            justify="left",
        )
        # get size of the label to use later for positioning and sizing of the tooltip
        x, y = label.winfo_reqwidth(), label.winfo_reqheight()
        # set the size of the tooltip background to be 1px larger than the label
        frame.configure(width=x + 1, height=y + 1)
        # offset the frame 1px from edge of the tooltip corner
        frame.place(x=1, y=1, anchor="nw")
        label.place(x=0, y=0, anchor="nw")
        root_x = (
            self.widget.winfo_rootx() + self.widget.winfo_width() + 4
        )  # offset the tooltip by extra 4px so it doesn't overlap the widget
        root_y = self.widget.winfo_rooty() - self.widget.winfo_height() - 4
        # set position of the tooltip, size and add 2px around the tooltip for a 1px border
        self.geometry(f"{x+2}x{y+2}+{root_x}+{root_y}")

    def hide(self):
        self.destroy()
